@@ -8,45 +8,62 @@ module MCollective
         def initialize(configfile)
             @config = Config.instance
             @config.loadconfig(configfile) unless @config.configured
-            @log = Log.instance
+
             @connection = PluginManager["connector_plugin"]
-
             @security = PluginManager["security_plugin"]
+
             @security.initiated_by = :client
-
             @options = nil
-
             @subscriptions = {}
 
             @connection.connect
         end
 
+        # Returns the configured main collective if no
+        # specific collective is specified as options
+        def collective
+            if @options[:collective].nil?
+                @config.main_collective
+            else
+                @options[:collective]
+            end
+        end
+
         # Disconnects cleanly from the middleware
         def disconnect
-            @log.debug("Disconnecting from the middleware")
+            Log.debug("Disconnecting from the middleware")
             @connection.disconnect
         end
 
         # Sends a request and returns the generated request id, doesn't wait for
         # responses and doesn't execute any passed in code blocks for responses
         def sendreq(msg, agent, filter = {})
-            target = Util.make_target(agent, :command)
+            target = Util.make_target(agent, :command, collective)
 
             reqid = Digest::MD5.hexdigest("#{@config.identity}-#{Time.now.to_f.to_s}-#{target}")
 
-            req = @security.encoderequest(@config.identity, target, msg, reqid, filter)
+            # Security plugins now accept an agent and collective, ones written for <= 1.1.4 dont
+            # but we still want to support them, try to call them in a compatible way if they
+            # dont support the new arguments
+            begin
+                req = @security.encoderequest(@config.identity, target, msg, reqid, filter, agent, collective)
+            rescue ArgumentError
+                req = @security.encoderequest(@config.identity, target, msg, reqid, filter)
+            end
 
-            @log.debug("Sending request #{reqid} to #{target}")
+            Log.debug("Sending request #{reqid} to #{target}")
 
             unless @subscriptions.include?(agent)
-                topic = Util.make_target(agent, :reply)
-                @log.debug("Subscribing to #{topic}")
+                topic = Util.make_target(agent, :reply, collective)
+                Log.debug("Subscribing to #{topic}")
 
-                @connection.subscribe(topic)
+                Util.subscribe(topic)
                 @subscriptions[agent] = 1
             end
 
-            @connection.send(target, req)
+            Timeout.timeout(2) do
+                @connection.send(target, req)
+            end
 
             reqid
         end
@@ -69,10 +86,10 @@ module MCollective
 
                 raise(MsgDoesNotMatchRequestID, "Message reqid #{requestid} does not match our reqid #{msg[:requestid]}") if msg[:requestid] != requestid
             rescue SecurityValidationFailed => e
-                @log.warn("Ignoring a message that did not pass security validations")
+                Log.warn("Ignoring a message that did not pass security validations")
                 retry
             rescue MsgDoesNotMatchRequestID => e
-                @log.debug("Ignoring a message for some other client")
+                Log.debug("Ignoring a message for some other client")
                 retry
             end
 
@@ -83,14 +100,14 @@ module MCollective
         # returns an array of nodes
         def discover(filter, timeout)
             begin
-                reqid = sendreq("ping", "discovery", filter)
-                @log.debug("Waiting #{timeout} seconds for discovery replies to request #{reqid}")
-
                 hosts = []
                 Timeout.timeout(timeout) do
+                    reqid = sendreq("ping", "discovery", filter)
+                    Log.debug("Waiting #{timeout} seconds for discovery replies to request #{reqid}")
+
                     loop do
                         msg = receive(reqid)
-                        @log.debug("Got discovery reply from #{msg[:senderid]}")
+                        Log.debug("Got discovery reply from #{msg[:senderid]}")
                         hosts << msg[:senderid]
                     end
                 end
@@ -116,12 +133,12 @@ module MCollective
 
             STDOUT.sync = true
 
-            reqid = sendreq(body, agent, options[:filter])
-
             hosts_responded = 0
 
             begin
                 Timeout.timeout(options[:timeout]) do
+                    reqid = sendreq(body, agent, options[:filter])
+
                     loop do
                         resp = receive(reqid)
 
@@ -178,10 +195,10 @@ module MCollective
 
             raise("No matching clients found") if discovered == 0
 
-            reqid = sendreq(body, agent, options[:filter])
-
             begin
                 Timeout.timeout(options[:timeout]) do
+                    reqid = sendreq(body, agent, options[:filter])
+
                     (1..discovered).each do |c|
                         resp = receive(reqid)
 
