@@ -7,35 +7,15 @@ module MCollective
                     :keeplogs, :max_log_size, :loglevel, :identity, :daemonize, :connector,
                     :securityprovider, :factsource, :registration, :registerinterval, :topicsep,
                     :classesfile, :rpcauditprovider, :rpcaudit, :configdir, :rpcauthprovider,
-                    :rpcauthorization, :color, :configfile, :rpchelptemplate, :rpclimitmethod
+                    :rpcauthorization, :color, :configfile, :rpchelptemplate, :rpclimitmethod,
+                    :logger_type, :fact_cache_time, :collectives, :main_collective, :ssl_cipher
 
         def initialize
             @configured = false
         end
 
         def loadconfig(configfile)
-            @stomp = Hash.new
-            @subscribe = Array.new
-            @pluginconf = Hash.new
-            @connector = "Stomp"
-            @securityprovider = "Psk"
-            @factsource = "Yaml"
-            @identity = Socket.gethostname
-            @registration = "Agentlist"
-            @registerinterval = 0
-            @topicsep = "."
-            @classesfile = "/var/lib/puppet/classes.txt"
-            @rpcaudit = false
-            @rpcauditprovider = ""
-            @rpcauthorization = false
-            @rpcauthprovider = ""
-            @configdir = File.dirname(configfile)
-            @color = true
-            @configfile = configfile
-            @rpchelptemplate = "/etc/mcollective/rpc-help.erb"
-            @keeplogs = 5
-            @max_log_size = 2097152
-            @rpclimitmethod = :first
+            set_config_defaults(configfile)
 
             if File.exists?(configfile)
                 File.open(configfile, "r").each do |line|
@@ -55,6 +35,10 @@ module MCollective
                                     @registration = val.capitalize
                                 when "registerinterval"
                                     @registerinterval = val.to_i
+                                when "collectives"
+                                    @collectives = val.split(",").map {|c| c.strip}
+                                when "main_collective"
+                                    @main_collective = val
                                 when "topicprefix"
                                     @topicprefix = val
                                 when "logfile"
@@ -66,9 +50,12 @@ module MCollective
                                 when "loglevel"
                                      @loglevel = val
                                 when "libdir"
-                                    @libdir = val
-                                    unless $LOAD_PATH.include?(val)
-                                        $LOAD_PATH << val
+                                    paths = val.split(/:/)
+                                    paths.each do |path|
+                                        @libdir << path
+                                        unless $LOAD_PATH.include?(path)
+                                            $LOAD_PATH << path
+                                        end
                                     end
                                 when "identity"
                                     @identity = val
@@ -98,7 +85,12 @@ module MCollective
                                     @rpchelptemplate = val
                                 when "rpclimitmethod"
                                     @rpclimitmethod = val.to_sym
-
+                                when "logger_type"
+                                    @logger_type = val
+                                when "fact_cache_time"
+                                    @fact_cache_time = val.to_i
+                                when "ssl_cipher"
+                                    @ssl_cipher = val
                                 else
                                     raise("Unknown config parameter #{key}")
                             end
@@ -106,9 +98,13 @@ module MCollective
                     end
                 end
 
+                read_plugin_config_dir("#{@configdir}/plugin.d")
+
                 @configured = true
 
-                PluginManager.loadclass("Mcollective::Facts::#{@factsource}")
+                @libdir.each {|dir| Log.warn("Cannot find libdir: #{dir}") unless File.directory?(dir)}
+
+                PluginManager.loadclass("Mcollective::Facts::#{@factsource}_facts")
                 PluginManager.loadclass("Mcollective::Connector::#{@connector}")
                 PluginManager.loadclass("Mcollective::Security::#{@securityprovider}")
                 PluginManager.loadclass("Mcollective::Registration::#{@registration}")
@@ -118,6 +114,58 @@ module MCollective
                 raise("Cannot find config file '#{configfile}'")
             end
         end
+
+        def set_config_defaults(configfile)
+            @stomp = Hash.new
+            @subscribe = Array.new
+            @pluginconf = Hash.new
+            @connector = "Stomp"
+            @securityprovider = "Psk"
+            @factsource = "Yaml"
+            @identity = Socket.gethostname
+            @registration = "Agentlist"
+            @registerinterval = 0
+            @topicsep = "."
+            @classesfile = "/var/lib/puppet/classes.txt"
+            @rpcaudit = false
+            @rpcauditprovider = ""
+            @rpcauthorization = false
+            @rpcauthprovider = ""
+            @configdir = File.dirname(configfile)
+            @color = true
+            @configfile = configfile
+            @rpchelptemplate = "/etc/mcollective/rpc-help.erb"
+            @logger_type = "file"
+            @keeplogs = 5
+            @max_log_size = 2097152
+            @rpclimitmethod = :first
+            @libdir = Array.new
+            @fact_cache_time = 300
+            @loglevel = "info"
+            @collectives = ["mcollective"]
+            @main_collective = @collectives.first
+            @ssl_cipher = "aes-256-cbc"
+        end
+
+        def read_plugin_config_dir(dir)
+            return unless File.directory?(dir)
+
+            Dir.new(dir).each do |pluginconfigfile|
+                next unless pluginconfigfile =~ /^([\w]+).cfg$/
+
+                plugin = $1
+                File.open("#{dir}/#{pluginconfigfile}", "r").each do |line|
+                    # strip blank lines
+                    line.gsub!(/\s*$/, "")
+                    next if line =~ /^#|^$/
+                    if (line =~ /(.+?)\s*=\s*(.+)/)
+                        key = $1
+                        val = $2
+                        @pluginconf["#{plugin}.#{key}"] = val
+                    end
+                end
+            end
+        end
     end
 end
 
