@@ -15,6 +15,8 @@ module MCollective
                 reply[:facts] = PluginManager["facts_plugin"].get_facts
                 reply[:version] = MCollective.version
                 reply[:classes] = []
+                reply[:main_collective] = config.main_collective
+                reply[:collectives] = config.collectives
 
                 cfile = Config.instance.classesfile
                 if File.exist?(cfile)
@@ -53,7 +55,15 @@ module MCollective
                     agent = PluginManager["#{target_agent}_agent"]
                     actions = agent.methods.grep(/_agent/)
 
-                    agent_data = {:agent => target_agent}
+                    agent_data = {:agent => target_agent,
+                                  :license => "unknown",
+                                  :timeout => agent.timeout,
+                                  :description => "unknown",
+                                  :name => target_agent,
+                                  :url => "unknown",
+                                  :version => "unknown",
+                                  :author => "unknown"}
+
                     agent_data.merge!(agent.meta)
 
                     reply[:agents] << agent_data
@@ -69,6 +79,18 @@ module MCollective
                 reply[:item] = request[:item]
                 reply[:value] = config.send(request[:item])
             end
+
+            # Responds to PING requests with the local timestamp
+            action "ping" do
+                reply[:pong] = Time.now.to_i
+            end
+
+            # Returns all configured collectives
+            action "collective_info" do
+                config = Config.instance
+                reply[:main_collective] = config.main_collective
+                reply[:collectives] = config.collectives
+            end
         end
     end
 end
